from django.core.signals import request_started
from django.db import close_old_connections
from django.test import SimpleTestCase, override_settings

from lgi import get_lgi_application


@override_settings(ROOT_URLCONF="urls")
class LGITestCase(SimpleTestCase):
    def setUp(self):
        request_started.disconnect(close_old_connections)
        self.application = get_lgi_application()

    def tearDown(self):
        request_started.connect(close_old_connections)

    def test_get_lgi_application_manage(self):
        event = {"manage": ["check"]}
        response = self.application(event, None)
        self.assertEqual(
            response, {"output": "System check identified no issues (0 silenced).\n"}
        )

    def test_get_lgi_application_gateway(self):
        event = {
            "version": "2.0",
            "rawQueryString": "",
            "headers": {},
            "requestContext": {
                "domainName": "id.execute-api.us-east-1.amazonaws.com",
                "http": {"method": "GET", "path": "/", "sourceIp": "IP"},
            },
        }
        response = self.application(event, None)
        self.assertEqual(response["body"], "Hello World!")
        self.assertEqual(
            response["headers"], {"Content-Type": "text/html; charset=utf-8"}
        )
        self.assertEqual(response["statusCode"], 200)

    def test_body(self):
        event = {
            "version": "2.0",
            "rawQueryString": "",
            "headers": {
                "Content-Type": "application/x-www-form-urlencoded; charset=utf-8"
            },
            "requestContext": {
                "domainName": "id.execute-api.us-east-1.amazonaws.com",
                "http": {"method": "POST", "path": "/", "sourceIp": "IP"},
            },
            "body": "name=Andrew",
        }
        response = self.application(event, None)
        self.assertEqual(response["body"], "Hello Andrew!")
        self.assertEqual(response["statusCode"], 200)

    def test_body_b64(self):
        event = {
            "version": "2.0",
            "rawQueryString": "",
            "headers": {
                "Content-Type": "application/x-www-form-urlencoded; charset=utf-8"
            },
            "requestContext": {
                "domainName": "id.execute-api.us-east-1.amazonaws.com",
                "http": {"method": "POST", "path": "/", "sourceIp": "IP"},
            },
            "body": "bmFtZT1BbmRyZXc=",  # name=Andrew
            "isBase64Encoded": True,
        }
        response = self.application(event, None)
        self.assertEqual(response["body"], "Hello Andrew!")
        self.assertEqual(response["statusCode"], 200)

    def test_get_query_string(self):
        event = {
            "version": "2.0",
            "rawQueryString": "name=Andrew",
            "headers": {},
            "requestContext": {
                "domainName": "id.execute-api.us-east-1.amazonaws.com",
                "http": {"method": "GET", "path": "/", "sourceIp": "IP"},
            },
        }
        response = self.application(event, None)
        self.assertEqual(response["body"], "Hello Andrew!")

    def test_headers(self):
        event = {
            "version": "2.0",
            "rawQueryString": "",
            "headers": {
                "Content-Type": "text/plain; charset=utf-8",
                "Referer": "Wales",
            },
            "requestContext": {
                "domainName": "id.execute-api.us-east-1.amazonaws.com",
                "http": {"method": "GET", "path": "/meta/", "sourceIp": "IP"},
            },
        }
        response = self.application(event, None)
        self.assertEqual(response["body"], "From Wales")
        self.assertEqual(
            response["headers"], {"Content-Type": "text/plain; charset=utf-8"}
        )
        self.assertEqual(response["statusCode"], 200)



#!/usr/bin/env python
import unittest

try:
    import lgi  # noqa: F401
except ImportError as e:
    raise RuntimeError(
        "django-lgi module not found, reference README.md for instructions."
    ) from e
else:
    from django.conf import settings

if __name__ == "__main__":
    settings.configure()
    unittest.main("tests")



"""
Django Lambda Gateway Interface
"""
__version__ = "0.1.2"
import logging
from base64 import b64decode
from io import BytesIO, StringIO

import django
from django.conf import settings
from django.core import management, signals
from django.core.handlers import base
from django.http import HttpRequest, QueryDict, parse_cookie
from django.urls import set_script_prefix
from django.utils.functional import cached_property

logger = logging.getLogger("django.request")
_default_route_key = "$default"


class LGIRequest(HttpRequest):
    def __init__(self, event):
        script_name = get_script_name(event)
        self.path = event["requestContext"]["http"]["path"]
        # If PATH_INFO is empty (e.g. accessing the SCRIPT_NAME URL without a
        # trailing slash), operate as if '/' was requested.
        prefix_end = len(script_name.rstrip("/"))
        self.path_info = self.path[prefix_end:] or "/"
        self.event = event
        self.method = event["requestContext"]["http"]["method"]
        self.META = {
            "REQUEST_METHOD": self.method,
            "QUERY_STRING": event["rawQueryString"],
            "SCRIPT_NAME": script_name,
            "PATH_INFO": self.path_info,
            "REMOTE_ADDR": event["requestContext"]["http"]["sourceIp"],
            "REMOTE_HOST": event["requestContext"]["http"]["sourceIp"],
            "SERVER_NAME": event["requestContext"]["domainName"],
            "SERVER_PORT": "443",
            "wsgi.multithread": False,
            "wsgi.multiprocess": False,
        }
        for name, value in event["headers"].items():
            corrected_name = name.replace("-", "_").upper()
            if corrected_name not in ("CONTENT_TYPE", "CONTENT_LENGTH"):
                corrected_name = f"HTTP_{corrected_name}"
            # Duplicate query strings are combined with commas
            self.META[corrected_name] = value
        self._set_content_type_params(self.META)
        body = self.event.get("body", "")
        if event.get("isBase64Encoded", False):
            body_bytes = b64decode(body)
        else:
            body_bytes = body.encode()
        self._stream = BytesIO(body_bytes)
        self._read_started = False
        self.resolver_match = None

    def _get_scheme(self):
        # Amazon API Gateway exposes HTTPS endpoints only
        return "https"

    @cached_property
    def GET(self):
        return QueryDict(self.META["QUERY_STRING"])

    def _get_post(self):
        if not hasattr(self, "_post"):
            self._load_post_and_files()
        return self._post

    def _set_post(self, post):
        self._post = post

    POST = property(_get_post, _set_post)

    @property
    def FILES(self):
        if not hasattr(self, "_files"):
            self._load_post_and_files()
        return self._files

    @cached_property
    def COOKIES(self):
        # parse_cookie expects cookies list as a ; separated string
        return parse_cookie(";".join(self.event.get("cookies", [])))


class LGIHandler(base.BaseHandler):
    request_class = LGIRequest

    def __init__(self):
        super().__init__()
        self.load_middleware()

    def __call__(self, event, context):
        logger.debug(event)

        # management commands
        if "manage" in event:
            output = StringIO()
            management.call_command(*event["manage"], stdout=output)
            return {"output": output.getvalue()}

        # api gateway
        version = event["version"]
        if version != "2.0":
            raise ValueError(f"{version} format version is not supported")

        set_script_prefix(get_script_name(event))
        signals.request_started.send(sender=self.__class__, event=event)
        request = self.request_class(event)
        response = self.get_response(request)

        response._handler_class = self.__class__

        return {
            "cookies": list(c.output(header="") for c in response.cookies.values()),
            "isBase64Encoded": False,
            "statusCode": response.status_code,
            "headers": dict(response.items()),
            "body": response.content.decode(),
        }


def get_script_name(event):
    return settings.FORCE_SCRIPT_NAME or ""


def get_lgi_application():
    django.setup(set_prefix=False)
    return LGIHandler()



from django.http import HttpResponse
from django.urls import path


def hello(request):
    if request.method == "GET":
        arg = request.GET.get("name")
    elif request.method == "POST":
        arg = request.POST.get("name")
    name = arg or "World"
    return HttpResponse("Hello %s!" % name)


def hello_meta(request):
    return HttpResponse(
        "From %s" % request.META.get("HTTP_REFERER") or "",
        content_type=request.META.get("CONTENT_TYPE"),
    )


urlpatterns = [
    path("", hello),
    path("meta/", hello_meta),
]



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

install_requires = \
['django>=3']

extras_require = \
{'dev': ['black', 'isort', 'pre-commit'], 'test': ['tox']}

setup(name='django-lgi',
      version='0.1.2',
      description='Django Lambda Gateway Interface',
      author='Jarosław Wygoda',
      author_email='jaroslaw@wygoda.me',
      url='https://github.com/jwygoda/django-lgi',
      py_modules=['lgi'],
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
